package org.bdware.mockjava;

import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonPrimitive;
import org.bdware.mockjava.generator.IntegerGenerator;

public class MockGenerator extends JsonVisitor {
    public static MockGenerator instance = new MockGenerator();
    JsonElement result;

    public JsonElement get() {
        return result;
    }

    @Override
    public JsonVisitor visitObject(JsonObject jsonObject) {
        JsonObject jo = new JsonObject();
        for (String key : jsonObject.keySet()) {
            JsonElement origin = jsonObject.get(key);
            visit(origin);
            JsonElement je = get();
            if (origin.isJsonArray()) {
                key += "|" + Math.min(((JsonArray) origin).size(), 20);
            }
            jo.add(key, je);
        }
        result = jo;
        return this;
    }

    @Override
    public JsonVisitor visitJsonArray(JsonArray array) {
        JsonArray ret = new JsonArray();
        if (array.size() > 0) {
            visit(array.get(0));
            ret.add(result);
        }
        result = ret;
        return this;
    }

    @Override
    public JsonVisitor visitPrimitive(JsonPrimitive primitive) {
        if (!primitive.isString()) result = primitive.deepCopy();

        ValueGenerator generator = lookupGenerator(primitive.getAsString());
        result = generator.generate(null, primitive);
        return this;
    }

    private ValueGenerator lookupGenerator(final String str) {
        if (!str.startsWith("@")) {
            return new ValueGenerator() {
                @Override
                public JsonElement generate(String preField, JsonElement previous) {
                    return new JsonPrimitive(str);
                }
            };
        }
        String[] args = null;
        int from = str.indexOf("(");
        if (from != -1) {
            int to = str.indexOf(")");
            String subStr = str.substring(from + 1, to);
            args = subStr.split(",");
        }
        return IntegerGenerator.create(args);
    }
}
